package thunder.jbosscluster;

import java.io.IOException;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.testng.annotations.Test;

// purpose the this test case is mainly to verify the transparent failover of HA-jndi itself and NOT the specific clustered services.
// the concerns is: is HA-jndi itself of capability of transparent failover?
// scenario: when you lookup a clustered ejb using HA-jndi, and do something, and then current server is down,
// what will happen if you perform another lookup using the same jndi context?
// will the context cache the information of living servers, is it smart enough to know that current server is down and failover transparently?

// 1. look up the clustered ejb and the non clustered ejb, call each of them.
// 2. suspend current thread
// 3. manually shutdown the server which the smart proxy is currently connecting to.
// 4. resume the current thread, look up the both ejbs again using cached jndi context.
// 5. both ejb should work fine
// actually ?
//

public class JBossClusteredHAJNDIFailoverTest {

	@Test
	public void testHAJNDIFailover() {

		InitialContext ctx = getInitialContext();

		ClusteredEJBRemote clusteredRemote = null;
		NonClusteredEJBRemote nonClusteredRemote = null;

		// 1. look up the clustered ejb and the non clustered ejb, call each of them.

		// clustered ejb
		try {
			clusteredRemote = HAJNDILookup(ClusteredEJBRemote.class, "ClusteredEJBMappedName", ctx);
			System.out.println("helloCluster = " + clusteredRemote.helloCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}
		// non clustered ejb
		try {
			nonClusteredRemote = HAJNDILookup(NonClusteredEJBRemote.class, "NonClusteredEJBMappedName", ctx);
			System.out.println("helloNonCluster = " + nonClusteredRemote.helloNonCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}

		System.out.println("========================================");
		System.out.println("pls shutdown the CURRENT working  server...");
		// 2. suspend current thread
		try {
			System.in.read();
		} catch (IOException e) {
			e.printStackTrace();
		}

		System.out.println("++++++++++++++++++++++++++++++++++++++++");

		// 4. resume the current thread, look up the both ejbs again using cached jndi context.

		// clustered ejb
		try {
			clusteredRemote = HAJNDILookup(ClusteredEJBRemote.class, "ClusteredEJBMappedName", ctx);
			System.out.println("helloCluster = " + clusteredRemote.helloCluster());
		} catch (NamingException nex) {
			System.out.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^");
			nex.printStackTrace();
			// this time, everything works fine because the ha-jndi proxy get updated from the last failure.
			try {
				System.out.println("after current server shutdown, lookup the clustered ejb for second timne, it should be ok");
				clusteredRemote = HAJNDILookup(ClusteredEJBRemote.class, "ClusteredEJBMappedName", ctx);
				System.out.println("helloCluster = " + clusteredRemote.helloCluster());
			} catch (NamingException nex2) {
				nex.printStackTrace();
			}
		}

		// non cluster ejb
		try {
			nonClusteredRemote = HAJNDILookup(NonClusteredEJBRemote.class, "NonClusteredEJBMappedName", ctx);
			System.out.println("helloNonCluster = " + nonClusteredRemote.helloNonCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}
	}

	private static <T> T HAJNDILookup(Class<T> businessInterfaceClass, String jndiName, InitialContext ctx) throws NamingException {

		T remoteBusinessObject = null;
		remoteBusinessObject = (T)ctx.lookup(jndiName);
		return remoteBusinessObject;
	}

	private static InitialContext getInitialContext() {

		Hashtable jndiEvn = new Hashtable();
		jndiEvn.put(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
		jndiEvn.put(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
		jndiEvn.put(Context.PROVIDER_URL, "jnp://10.28.71.51:1100,jnp://10.28.71.188:1100");
		jndiEvn.put("jnp.disableDiscovery", "true");
		InitialContext ctx = null;
		try {
			ctx = new InitialContext(jndiEvn);
		} catch (NamingException e) {
			e.printStackTrace();
		}
		return ctx;

	}
}