package thunder.jbosscluster;

import java.io.IOException;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.testng.annotations.Test;

// if non ha-jndi works with a clustered EJB
public class JBossNonHAJndiAndClusteredEJBTest {

	@Test
	public void testNonHAJNDIANdClusteredEJB() {

		InitialContext ctx = getNonHaInitialContext();

		ClusteredEJBRemote clusteredRemote = null;
		NonClusteredEJBRemote nonClusteredRemote = null;

		// 1. look up the clustered ejb and the non clustered ejb, call each of them.
		// wrap two calls in different try - catch block
		// clustered
		try {
			clusteredRemote = noHAJNDILookup(ClusteredEJBRemote.class, "ClusteredEJBMappedName", ctx);
			System.out.println("helloCluster = " + clusteredRemote.helloCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}
		// non clustered
		try {
			nonClusteredRemote = noHAJNDILookup(NonClusteredEJBRemote.class, "NonClusteredEJBMappedName", ctx);
			System.out.println("helloNonCluster = " + nonClusteredRemote.helloNonCluster());
		} catch (NamingException nex) {
			nex.printStackTrace();
		}

		System.out.println("========================================");
		System.out.println("pls shutdown the CURRENT working server...");
		// 2. suspend current thread
		try {
			System.in.read();
		} catch (IOException e) {
			e.printStackTrace();
		}

		System.out.println("++++++++++++++++++++++++++++++++++++++++");

		// 4. resume the current thread, calling the both ejbs with cached ejb references.

		// clustered
		System.out.println("helloCluster = " + clusteredRemote.helloCluster());

		// non clustered
		try {
			System.out.println("helloNonCluster = " + nonClusteredRemote.helloNonCluster());
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private static <T> T noHAJNDILookup(Class<T> businessInterfaceClass, String jndiName, InitialContext ctx) throws NamingException {

		T remoteBusinessObject = null;
		remoteBusinessObject = (T)ctx.lookup(jndiName);
		return remoteBusinessObject;
	}

	private static InitialContext getNonHaInitialContext() {

		Hashtable jndiEvn = new Hashtable();
		jndiEvn.put(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
		jndiEvn.put(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
		jndiEvn.put(Context.PROVIDER_URL, "jnp://10.28.71.188:1099");
		jndiEvn.put("jnp.disableDiscovery", "true");

		InitialContext ctx = null;
		try {
			ctx = new InitialContext(jndiEvn);
		} catch (NamingException e) {
			e.printStackTrace();
		}
		return ctx;

	}

}